#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_switch_controller_managed_switch
short_description: Configure FortiSwitch devices that are managed by this FortiGate in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify switch_controller feature and managed_switch category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    switch_controller_managed_switch:
        description:
            - Configure FortiSwitch devices that are managed by this FortiGate.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            802-1X-settings:
                description:
                    - Configuration method to edit FortiSwitch 802.1X global settings.
                suboptions:
                    link-down-auth:
                        description:
                            - Authentication state to set if a link is down.
                        choices:
                            - set-unauth
                            - no-action
                    local-override:
                        description:
                            - Enable to override global 802.1X settings on individual FortiSwitches.
                        choices:
                            - enable
                            - disable
                    max-reauth-attempt:
                        description:
                            - Maximum number of authentication attempts (0 - 15, default = 3).
                    reauth-period:
                        description:
                            - Reauthentication time interval (1 - 1440 min, default = 60, 0 = disable).
            connected:
                description:
                    - CAPWAP connection.
            custom-command:
                description:
                    - Configuration method to edit FortiSwitch commands to be pushed to this FortiSwitch device upon rebooting the FortiGate switch controller
                       or the FortiSwitch.
                suboptions:
                    command-entry:
                        description:
                            - List of FortiSwitch commands.
                        required: true
                    command-name:
                        description:
                            - Names of commands to be pushed to this FortiSwitch device, as configured under config switch-controller custom-command. Source
                               switch-controller.custom-command.command-name.
            delayed-restart-trigger:
                description:
                    - Delayed restart triggered for this FortiSwitch.
            description:
                description:
                    - Description.
            directly-connected:
                description:
                    - Directly connected FortiSwitch.
            dynamic-capability:
                description:
                    - List of features this FortiSwitch supports (not configurable) that is sent to the FortiGate device for subsequent configuration
                       initiated by the FortiGate device.
            dynamically-discovered:
                description:
                    - Dynamically discovered FortiSwitch.
            fsw-wan1-admin:
                description:
                    - FortiSwitch WAN1 admin status; enable to authorize the FortiSwitch as a managed switch.
                choices:
                    - discovered
                    - disable
                    - enable
            fsw-wan1-peer:
                description:
                    - Fortiswitch WAN1 peer port.
            fsw-wan2-admin:
                description:
                    - FortiSwitch WAN2 admin status; enable to authorize the FortiSwitch as a managed switch.
                choices:
                    - discovered
                    - disable
                    - enable
            fsw-wan2-peer:
                description:
                    - FortiSwitch WAN2 peer port.
            igmp-snooping:
                description:
                    - Configure FortiSwitch IGMP snooping global settings.
                suboptions:
                    aging-time:
                        description:
                            - Maximum time to retain a multicast snooping entry for which no packets have been seen (15 - 3600 sec, default = 300).
                    flood-unknown-multicast:
                        description:
                            - Enable/disable unknown multicast flooding.
                        choices:
                            - enable
                            - disable
                    local-override:
                        description:
                            - Enable/disable overriding the global IGMP snooping configuration.
                        choices:
                            - enable
                            - disable
            max-allowed-trunk-members:
                description:
                    - FortiSwitch maximum allowed trunk members.
            mirror:
                description:
                    - Configuration method to edit FortiSwitch packet mirror.
                suboptions:
                    dst:
                        description:
                            - Destination port.
                    name:
                        description:
                            - Mirror name.
                        required: true
                    src-egress:
                        description:
                            - Source egress interfaces.
                        suboptions:
                            name:
                                description:
                                    - Interface name.
                                required: true
                    src-ingress:
                        description:
                            - Source ingress interfaces.
                        suboptions:
                            name:
                                description:
                                    - Interface name.
                                required: true
                    status:
                        description:
                            - Active/inactive mirror configuration.
                        choices:
                            - active
                            - inactive
                    switching-packet:
                        description:
                            - Enable/disable switching functionality when mirroring.
                        choices:
                            - enable
                            - disable
            name:
                description:
                    - Managed-switch name.
            owner-vdom:
                description:
                    - VDOM which owner of port belongs to.
            poe-pre-standard-detection:
                description:
                    - Enable/disable PoE pre-standard detection.
                choices:
                    - enable
                    - disable
            ports:
                description:
                    - Managed-switch port list.
                suboptions:
                    allowed-vlans:
                        description:
                            - Configure switch port tagged vlans
                        suboptions:
                            vlan-name:
                                description:
                                    - VLAN name. Source system.interface.name.
                                required: true
                    allowed-vlans-all:
                        description:
                            - Enable/disable all defined vlans on this port.
                        choices:
                            - enable
                            - disable
                    arp-inspection-trust:
                        description:
                            - Trusted or untrusted dynamic ARP inspection.
                        choices:
                            - untrusted
                            - trusted
                    bundle:
                        description:
                            - Enable/disable Link Aggregation Group (LAG) bundling for non-FortiLink interfaces.
                        choices:
                            - enable
                            - disable
                    description:
                        description:
                            - Description for port.
                    dhcp-snoop-option82-trust:
                        description:
                            - Enable/disable allowance of DHCP with option-82 on untrusted interface.
                        choices:
                            - enable
                            - disable
                    dhcp-snooping:
                        description:
                            - Trusted or untrusted DHCP-snooping interface.
                        choices:
                            - untrusted
                            - trusted
                    discard-mode:
                        description:
                            - Configure discard mode for port.
                        choices:
                            - none
                            - all-untagged
                            - all-tagged
                    edge-port:
                        description:
                            - Enable/disable this interface as an edge port, bridging connections between workstations and/or computers.
                        choices:
                            - enable
                            - disable
                    export-tags:
                        description:
                            - Switch controller export tag name.
                        suboptions:
                            tag-name:
                                description:
                                    - Switch tag name. Source switch-controller.switch-interface-tag.name.
                                required: true
                    export-to:
                        description:
                            - Export managed-switch port to a tenant VDOM. Source system.vdom.name.
                    export-to-pool:
                        description:
                            - Switch controller export port to pool-list. Source switch-controller.virtual-port-pool.name.
                    export-to-pool_flag:
                        description:
                            - Switch controller export port to pool-list.
                    fgt-peer-device-name:
                        description:
                            - FGT peer device name.
                    fgt-peer-port-name:
                        description:
                            - FGT peer port name.
                    fiber-port:
                        description:
                            - Fiber-port.
                    flags:
                        description:
                            - Port properties flags.
                    fortilink-port:
                        description:
                            - FortiLink uplink port.
                    igmp-snooping:
                        description:
                            - Set IGMP snooping mode for the physical port interface.
                        choices:
                            - enable
                            - disable
                    igmps-flood-reports:
                        description:
                            - Enable/disable flooding of IGMP reports to this interface when igmp-snooping enabled.
                        choices:
                            - enable
                            - disable
                    igmps-flood-traffic:
                        description:
                            - Enable/disable flooding of IGMP snooping traffic to this interface.
                        choices:
                            - enable
                            - disable
                    isl-local-trunk-name:
                        description:
                            - ISL local trunk name.
                    isl-peer-device-name:
                        description:
                            - ISL peer device name.
                    isl-peer-port-name:
                        description:
                            - ISL peer port name.
                    lacp-speed:
                        description:
                            - end Link Aggregation Control Protocol (LACP) messages every 30 seconds (slow) or every second (fast).
                        choices:
                            - slow
                            - fast
                    learning-limit:
                        description:
                            - Limit the number of dynamic MAC addresses on this Port (1 - 128, 0 = no limit, default).
                    lldp-profile:
                        description:
                            - LLDP port TLV profile. Source switch-controller.lldp-profile.name.
                    lldp-status:
                        description:
                            - LLDP transmit and receive status.
                        choices:
                            - disable
                            - rx-only
                            - tx-only
                            - tx-rx
                    loop-guard:
                        description:
                            - Enable/disable loop-guard on this interface, an STP optimization used to prevent network loops.
                        choices:
                            - enabled
                            - disabled
                    loop-guard-timeout:
                        description:
                            - Loop-guard timeout (0 - 120 min, default = 45).
                    max-bundle:
                        description:
                            - Maximum size of LAG bundle (1 - 24, default = 24)
                    mclag:
                        description:
                            - Enable/disable multi-chassis link aggregation (MCLAG).
                        choices:
                            - enable
                            - disable
                    member-withdrawal-behavior:
                        description:
                            - Port behavior after it withdraws because of loss of control packets.
                        choices:
                            - forward
                            - block
                    members:
                        description:
                            - Aggregated LAG bundle interfaces.
                        suboptions:
                            member-name:
                                description:
                                    - Interface name from available options.
                                required: true
                    min-bundle:
                        description:
                            - Minimum size of LAG bundle (1 - 24, default = 1)
                    mode:
                        description:
                            - "LACP mode: ignore and do not send control messages, or negotiate 802.3ad aggregation passively or actively."
                        choices:
                            - static
                            - lacp-passive
                            - lacp-active
                    poe-capable:
                        description:
                            - PoE capable.
                    poe-pre-standard-detection:
                        description:
                            - Enable/disable PoE pre-standard detection.
                        choices:
                            - enable
                            - disable
                    poe-status:
                        description:
                            - Enable/disable PoE status.
                        choices:
                            - enable
                            - disable
                    port-name:
                        description:
                            - Switch port name.
                        required: true
                    port-number:
                        description:
                            - Port number.
                    port-owner:
                        description:
                            - Switch port name.
                    port-prefix-type:
                        description:
                            - Port prefix type.
                    port-security-policy:
                        description:
                            - Switch controller authentication policy to apply to this managed switch from available options. Source switch-controller
                              .security-policy.802-1X.name switch-controller.security-policy.captive-portal.name.
                    port-selection-criteria:
                        description:
                            - Algorithm for aggregate port selection.
                        choices:
                            - src-mac
                            - dst-mac
                            - src-dst-mac
                            - src-ip
                            - dst-ip
                            - src-dst-ip
                    qos-policy:
                        description:
                            - Switch controller QoS policy from available options. Source switch-controller.qos.qos-policy.name.
                    sample-direction:
                        description:
                            - sFlow sample direction.
                        choices:
                            - tx
                            - rx
                            - both
                    sflow-counter-interval:
                        description:
                            - sFlow sampler counter polling interval (1 - 255 sec).
                    sflow-sample-rate:
                        description:
                            - sFlow sampler sample rate (0 - 99999 p/sec).
                    sflow-sampler:
                        description:
                            - Enable/disable sFlow protocol on this interface.
                        choices:
                            - enabled
                            - disabled
                    speed:
                        description:
                            - Switch port speed; default and available settings depend on hardware.
                        choices:
                            - 10half
                            - 10full
                            - 100half
                            - 100full
                            - 1000auto
                            - 1000fiber
                            - 1000full
                            - 10000
                            - 40000
                            - auto
                            - auto-module
                            - 100FX-half
                            - 100FX-full
                            - 100000full
                            - 2500full
                            - 25000full
                            - 50000full
                    speed-mask:
                        description:
                            - Switch port speed mask.
                    stacking-port:
                        description:
                            - Stacking port.
                    status:
                        description:
                            - "Switch port admin status: up or down."
                        choices:
                            - up
                            - down
                    stp-bpdu-guard:
                        description:
                            - Enable/disable STP BPDU guard on this interface.
                        choices:
                            - enabled
                            - disabled
                    stp-bpdu-guard-timeout:
                        description:
                            - BPDU Guard disabling protection (0 - 120 min).
                    stp-root-guard:
                        description:
                            - Enable/disable STP root guard on this interface.
                        choices:
                            - enabled
                            - disabled
                    stp-state:
                        description:
                            - Enable/disable Spanning Tree Protocol (STP) on this interface.
                        choices:
                            - enabled
                            - disabled
                    switch-id:
                        description:
                            - Switch id.
                    type:
                        description:
                            - "Interface type: physical or trunk port."
                        choices:
                            - physical
                            - trunk
                    untagged-vlans:
                        description:
                            - Configure switch port untagged vlans
                        suboptions:
                            vlan-name:
                                description:
                                    - VLAN name. Source system.interface.name.
                                required: true
                    virtual-port:
                        description:
                            - Virtualized switch port.
                    vlan:
                        description:
                            - Assign switch ports to a VLAN. Source system.interface.name.
            pre-provisioned:
                description:
                    - Pre-provisioned managed switch.
            staged-image-version:
                description:
                    - Staged image version for FortiSwitch.
            storm-control:
                description:
                    - Configuration method to edit FortiSwitch storm control for measuring traffic activity using data rates to prevent traffic disruption.
                suboptions:
                    broadcast:
                        description:
                            - Enable/disable storm control to drop broadcast traffic.
                        choices:
                            - enable
                            - disable
                    local-override:
                        description:
                            - Enable to override global FortiSwitch storm control settings for this FortiSwitch.
                        choices:
                            - enable
                            - disable
                    rate:
                        description:
                            - Rate in packets per second at which storm traffic is controlled (1 - 10000000, default = 500). Storm control drops excess
                               traffic data rates beyond this threshold.
                    unknown-multicast:
                        description:
                            - Enable/disable storm control to drop unknown multicast traffic.
                        choices:
                            - enable
                            - disable
                    unknown-unicast:
                        description:
                            - Enable/disable storm control to drop unknown unicast traffic.
                        choices:
                            - enable
                            - disable
            stp-settings:
                description:
                    - Configuration method to edit Spanning Tree Protocol (STP) settings used to prevent bridge loops.
                suboptions:
                    forward-time:
                        description:
                            - Period of time a port is in listening and learning state (4 - 30 sec, default = 15).
                    hello-time:
                        description:
                            - Period of time between successive STP frame Bridge Protocol Data Units (BPDUs) sent on a port (1 - 10 sec, default = 2).
                    local-override:
                        description:
                            - Enable to configure local STP settings that override global STP settings.
                        choices:
                            - enable
                            - disable
                    max-age:
                        description:
                            - Maximum time before a bridge port saves its configuration BPDU information (6 - 40 sec, default = 20).
                    max-hops:
                        description:
                            - Maximum number of hops between the root bridge and the furthest bridge (1- 40, default = 20).
                    name:
                        description:
                            - Name of local STP settings configuration.
                    pending-timer:
                        description:
                            - Pending time (1 - 15 sec, default = 4).
                    revision:
                        description:
                            - STP revision number (0 - 65535).
                    status:
                        description:
                            - Enable/disable STP.
                        choices:
                            - enable
                            - disable
            switch-device-tag:
                description:
                    - User definable label/tag.
            switch-id:
                description:
                    - Managed-switch id.
                required: true
            switch-log:
                description:
                    - Configuration method to edit FortiSwitch logging settings (logs are transferred to and inserted into the FortiGate event log).
                suboptions:
                    local-override:
                        description:
                            - Enable to configure local logging settings that override global logging settings.
                        choices:
                            - enable
                            - disable
                    severity:
                        description:
                            - Severity of FortiSwitch logs that are added to the FortiGate event log.
                        choices:
                            - emergency
                            - alert
                            - critical
                            - error
                            - warning
                            - notification
                            - information
                            - debug
                    status:
                        description:
                            - Enable/disable adding FortiSwitch logs to the FortiGate event log.
                        choices:
                            - enable
                            - disable
            switch-profile:
                description:
                    - FortiSwitch profile. Source switch-controller.switch-profile.name.
            switch-stp-settings:
                description:
                    - Configure spanning tree protocol (STP).
                suboptions:
                    status:
                        description:
                            - Enable/disable STP.
                        choices:
                            - enable
                            - disable
            type:
                description:
                    - Indication of switch type, physical or virtual.
                choices:
                    - virtual
                    - physical
            version:
                description:
                    - FortiSwitch version.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure FortiSwitch devices that are managed by this FortiGate.
    fortios_switch_controller_managed_switch:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      switch_controller_managed_switch:
        state: "present"
        802-1X-settings:
            link-down-auth: "set-unauth"
            local-override: "enable"
            max-reauth-attempt: "6"
            reauth-period: "7"
        connected: "8"
        custom-command:
         -
            command-entry: "<your_own_value>"
            command-name: "<your_own_value> (source switch-controller.custom-command.command-name)"
        delayed-restart-trigger: "12"
        description: "<your_own_value>"
        directly-connected: "14"
        dynamic-capability: "15"
        dynamically-discovered: "16"
        fsw-wan1-admin: "discovered"
        fsw-wan1-peer: "<your_own_value>"
        fsw-wan2-admin: "discovered"
        fsw-wan2-peer: "<your_own_value>"
        igmp-snooping:
            aging-time: "22"
            flood-unknown-multicast: "enable"
            local-override: "enable"
        max-allowed-trunk-members: "25"
        mirror:
         -
            dst: "<your_own_value>"
            name: "default_name_28"
            src-egress:
             -
                name: "default_name_30"
            src-ingress:
             -
                name: "default_name_32"
            status: "active"
            switching-packet: "enable"
        name: "default_name_35"
        owner-vdom: "<your_own_value>"
        poe-pre-standard-detection: "enable"
        ports:
         -
            allowed-vlans:
             -
                vlan-name: "<your_own_value> (source system.interface.name)"
            allowed-vlans-all: "enable"
            arp-inspection-trust: "untrusted"
            bundle: "enable"
            description: "<your_own_value>"
            dhcp-snoop-option82-trust: "enable"
            dhcp-snooping: "untrusted"
            discard-mode: "none"
            edge-port: "enable"
            export-tags:
             -
                tag-name: "<your_own_value> (source switch-controller.switch-interface-tag.name)"
            export-to: "<your_own_value> (source system.vdom.name)"
            export-to-pool: "<your_own_value> (source switch-controller.virtual-port-pool.name)"
            export-to-pool_flag: "53"
            fgt-peer-device-name: "<your_own_value>"
            fgt-peer-port-name: "<your_own_value>"
            fiber-port: "56"
            flags: "57"
            fortilink-port: "58"
            igmp-snooping: "enable"
            igmps-flood-reports: "enable"
            igmps-flood-traffic: "enable"
            isl-local-trunk-name: "<your_own_value>"
            isl-peer-device-name: "<your_own_value>"
            isl-peer-port-name: "<your_own_value>"
            lacp-speed: "slow"
            learning-limit: "66"
            lldp-profile: "<your_own_value> (source switch-controller.lldp-profile.name)"
            lldp-status: "disable"
            loop-guard: "enabled"
            loop-guard-timeout: "70"
            max-bundle: "71"
            mclag: "enable"
            member-withdrawal-behavior: "forward"
            members:
             -
                member-name: "<your_own_value>"
            min-bundle: "76"
            mode: "static"
            poe-capable: "78"
            poe-pre-standard-detection: "enable"
            poe-status: "enable"
            port-name: "<your_own_value>"
            port-number: "82"
            port-owner: "<your_own_value>"
            port-prefix-type: "84"
            port-security-policy: "<your_own_value> (source switch-controller.security-policy.802-1X.name switch-controller.security-policy.captive-portal
              .name)"
            port-selection-criteria: "src-mac"
            qos-policy: "<your_own_value> (source switch-controller.qos.qos-policy.name)"
            sample-direction: "tx"
            sflow-counter-interval: "89"
            sflow-sample-rate: "90"
            sflow-sampler: "enabled"
            speed: "10half"
            speed-mask: "93"
            stacking-port: "94"
            status: "up"
            stp-bpdu-guard: "enabled"
            stp-bpdu-guard-timeout: "97"
            stp-root-guard: "enabled"
            stp-state: "enabled"
            switch-id: "<your_own_value>"
            type: "physical"
            untagged-vlans:
             -
                vlan-name: "<your_own_value> (source system.interface.name)"
            virtual-port: "104"
            vlan: "<your_own_value> (source system.interface.name)"
        pre-provisioned: "106"
        staged-image-version: "<your_own_value>"
        storm-control:
            broadcast: "enable"
            local-override: "enable"
            rate: "111"
            unknown-multicast: "enable"
            unknown-unicast: "enable"
        stp-settings:
            forward-time: "115"
            hello-time: "116"
            local-override: "enable"
            max-age: "118"
            max-hops: "119"
            name: "default_name_120"
            pending-timer: "121"
            revision: "122"
            status: "enable"
        switch-device-tag: "<your_own_value>"
        switch-id: "<your_own_value>"
        switch-log:
            local-override: "enable"
            severity: "emergency"
            status: "enable"
        switch-profile: "<your_own_value> (source switch-controller.switch-profile.name)"
        switch-stp-settings:
            status: "enable"
        type: "virtual"
        version: "134"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_switch_controller_managed_switch_data(json):
    option_list = ['802-1X-settings', 'connected', 'custom-command',
                   'delayed-restart-trigger', 'description', 'directly-connected',
                   'dynamic-capability', 'dynamically-discovered', 'fsw-wan1-admin',
                   'fsw-wan1-peer', 'fsw-wan2-admin', 'fsw-wan2-peer',
                   'igmp-snooping', 'max-allowed-trunk-members', 'mirror',
                   'name', 'owner-vdom', 'poe-pre-standard-detection',
                   'ports', 'pre-provisioned', 'staged-image-version',
                   'storm-control', 'stp-settings', 'switch-device-tag',
                   'switch-id', 'switch-log', 'switch-profile',
                   'switch-stp-settings', 'type', 'version']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def switch_controller_managed_switch(data, fos):
    vdom = data['vdom']
    switch_controller_managed_switch_data = data['switch_controller_managed_switch']
    flattened_data = flatten_multilists_attributes(switch_controller_managed_switch_data)
    filtered_data = filter_switch_controller_managed_switch_data(flattened_data)
    if switch_controller_managed_switch_data['state'] == "present":
        return fos.set('switch-controller',
                       'managed-switch',
                       data=filtered_data,
                       vdom=vdom)

    elif switch_controller_managed_switch_data['state'] == "absent":
        return fos.delete('switch-controller',
                          'managed-switch',
                          mkey=filtered_data['switch-id'],
                          vdom=vdom)


def fortios_switch_controller(data, fos):
    login(data)

    if data['switch_controller_managed_switch']:
        resp = switch_controller_managed_switch(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "switch_controller_managed_switch": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "802-1X-settings": {"required": False, "type": "dict",
                                    "options": {
                                        "link-down-auth": {"required": False, "type": "str",
                                                           "choices": ["set-unauth", "no-action"]},
                                        "local-override": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                        "max-reauth-attempt": {"required": False, "type": "int"},
                                        "reauth-period": {"required": False, "type": "int"}
                                    }},
                "connected": {"required": False, "type": "int"},
                "custom-command": {"required": False, "type": "list",
                                   "options": {
                                       "command-entry": {"required": True, "type": "str"},
                                       "command-name": {"required": False, "type": "str"}
                                   }},
                "delayed-restart-trigger": {"required": False, "type": "int"},
                "description": {"required": False, "type": "str"},
                "directly-connected": {"required": False, "type": "int"},
                "dynamic-capability": {"required": False, "type": "int"},
                "dynamically-discovered": {"required": False, "type": "int"},
                "fsw-wan1-admin": {"required": False, "type": "str",
                                   "choices": ["discovered", "disable", "enable"]},
                "fsw-wan1-peer": {"required": False, "type": "str"},
                "fsw-wan2-admin": {"required": False, "type": "str",
                                   "choices": ["discovered", "disable", "enable"]},
                "fsw-wan2-peer": {"required": False, "type": "str"},
                "igmp-snooping": {"required": False, "type": "dict",
                                  "options": {
                                      "aging-time": {"required": False, "type": "int"},
                                      "flood-unknown-multicast": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                      "local-override": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]}
                                  }},
                "max-allowed-trunk-members": {"required": False, "type": "int"},
                "mirror": {"required": False, "type": "list",
                           "options": {
                               "dst": {"required": False, "type": "str"},
                               "name": {"required": True, "type": "str"},
                               "src-egress": {"required": False, "type": "list",
                                              "options": {
                                                  "name": {"required": True, "type": "str"}
                                              }},
                               "src-ingress": {"required": False, "type": "list",
                                               "options": {
                                                   "name": {"required": True, "type": "str"}
                                               }},
                               "status": {"required": False, "type": "str",
                                          "choices": ["active", "inactive"]},
                               "switching-packet": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]}
                           }},
                "name": {"required": False, "type": "str"},
                "owner-vdom": {"required": False, "type": "str"},
                "poe-pre-standard-detection": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "ports": {"required": False, "type": "list",
                          "options": {
                              "allowed-vlans": {"required": False, "type": "list",
                                                "options": {
                                                    "vlan-name": {"required": True, "type": "str"}
                                                }},
                              "allowed-vlans-all": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                              "arp-inspection-trust": {"required": False, "type": "str",
                                                       "choices": ["untrusted", "trusted"]},
                              "bundle": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                              "description": {"required": False, "type": "str"},
                              "dhcp-snoop-option82-trust": {"required": False, "type": "str",
                                                            "choices": ["enable", "disable"]},
                              "dhcp-snooping": {"required": False, "type": "str",
                                                "choices": ["untrusted", "trusted"]},
                              "discard-mode": {"required": False, "type": "str",
                                               "choices": ["none", "all-untagged", "all-tagged"]},
                              "edge-port": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                              "export-tags": {"required": False, "type": "list",
                                              "options": {
                                                  "tag-name": {"required": True, "type": "str"}
                                              }},
                              "export-to": {"required": False, "type": "str"},
                              "export-to-pool": {"required": False, "type": "str"},
                              "export-to-pool_flag": {"required": False, "type": "int"},
                              "fgt-peer-device-name": {"required": False, "type": "str"},
                              "fgt-peer-port-name": {"required": False, "type": "str"},
                              "fiber-port": {"required": False, "type": "int"},
                              "flags": {"required": False, "type": "int"},
                              "fortilink-port": {"required": False, "type": "int"},
                              "igmp-snooping": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                              "igmps-flood-reports": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                              "igmps-flood-traffic": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                              "isl-local-trunk-name": {"required": False, "type": "str"},
                              "isl-peer-device-name": {"required": False, "type": "str"},
                              "isl-peer-port-name": {"required": False, "type": "str"},
                              "lacp-speed": {"required": False, "type": "str",
                                             "choices": ["slow", "fast"]},
                              "learning-limit": {"required": False, "type": "int"},
                              "lldp-profile": {"required": False, "type": "str"},
                              "lldp-status": {"required": False, "type": "str",
                                              "choices": ["disable", "rx-only", "tx-only",
                                                          "tx-rx"]},
                              "loop-guard": {"required": False, "type": "str",
                                             "choices": ["enabled", "disabled"]},
                              "loop-guard-timeout": {"required": False, "type": "int"},
                              "max-bundle": {"required": False, "type": "int"},
                              "mclag": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                              "member-withdrawal-behavior": {"required": False, "type": "str",
                                                             "choices": ["forward", "block"]},
                              "members": {"required": False, "type": "list",
                                          "options": {
                                              "member-name": {"required": True, "type": "str"}
                                          }},
                              "min-bundle": {"required": False, "type": "int"},
                              "mode": {"required": False, "type": "str",
                                       "choices": ["static", "lacp-passive", "lacp-active"]},
                              "poe-capable": {"required": False, "type": "int"},
                              "poe-pre-standard-detection": {"required": False, "type": "str",
                                                             "choices": ["enable", "disable"]},
                              "poe-status": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                              "port-name": {"required": True, "type": "str"},
                              "port-number": {"required": False, "type": "int"},
                              "port-owner": {"required": False, "type": "str"},
                              "port-prefix-type": {"required": False, "type": "int"},
                              "port-security-policy": {"required": False, "type": "str"},
                              "port-selection-criteria": {"required": False, "type": "str",
                                                          "choices": ["src-mac", "dst-mac", "src-dst-mac",
                                                                      "src-ip", "dst-ip", "src-dst-ip"]},
                              "qos-policy": {"required": False, "type": "str"},
                              "sample-direction": {"required": False, "type": "str",
                                                   "choices": ["tx", "rx", "both"]},
                              "sflow-counter-interval": {"required": False, "type": "int"},
                              "sflow-sample-rate": {"required": False, "type": "int"},
                              "sflow-sampler": {"required": False, "type": "str",
                                                "choices": ["enabled", "disabled"]},
                              "speed": {"required": False, "type": "str",
                                        "choices": ["10half", "10full", "100half",
                                                    "100full", "1000auto", "1000fiber",
                                                    "1000full", "10000", "40000",
                                                    "auto", "auto-module", "100FX-half",
                                                    "100FX-full", "100000full", "2500full",
                                                    "25000full", "50000full"]},
                              "speed-mask": {"required": False, "type": "int"},
                              "stacking-port": {"required": False, "type": "int"},
                              "status": {"required": False, "type": "str",
                                         "choices": ["up", "down"]},
                              "stp-bpdu-guard": {"required": False, "type": "str",
                                                 "choices": ["enabled", "disabled"]},
                              "stp-bpdu-guard-timeout": {"required": False, "type": "int"},
                              "stp-root-guard": {"required": False, "type": "str",
                                                 "choices": ["enabled", "disabled"]},
                              "stp-state": {"required": False, "type": "str",
                                            "choices": ["enabled", "disabled"]},
                              "switch-id": {"required": False, "type": "str"},
                              "type": {"required": False, "type": "str",
                                       "choices": ["physical", "trunk"]},
                              "untagged-vlans": {"required": False, "type": "list",
                                                 "options": {
                                                     "vlan-name": {"required": True, "type": "str"}
                                                 }},
                              "virtual-port": {"required": False, "type": "int"},
                              "vlan": {"required": False, "type": "str"}
                          }},
                "pre-provisioned": {"required": False, "type": "int"},
                "staged-image-version": {"required": False, "type": "str"},
                "storm-control": {"required": False, "type": "dict",
                                  "options": {
                                      "broadcast": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                                      "local-override": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                      "rate": {"required": False, "type": "int"},
                                      "unknown-multicast": {"required": False, "type": "str",
                                                            "choices": ["enable", "disable"]},
                                      "unknown-unicast": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]}
                                  }},
                "stp-settings": {"required": False, "type": "dict",
                                 "options": {
                                     "forward-time": {"required": False, "type": "int"},
                                     "hello-time": {"required": False, "type": "int"},
                                     "local-override": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                     "max-age": {"required": False, "type": "int"},
                                     "max-hops": {"required": False, "type": "int"},
                                     "name": {"required": False, "type": "str"},
                                     "pending-timer": {"required": False, "type": "int"},
                                     "revision": {"required": False, "type": "int"},
                                     "status": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]}
                                 }},
                "switch-device-tag": {"required": False, "type": "str"},
                "switch-id": {"required": True, "type": "str"},
                "switch-log": {"required": False, "type": "dict",
                               "options": {
                                   "local-override": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                   "severity": {"required": False, "type": "str",
                                                "choices": ["emergency", "alert", "critical",
                                                            "error", "warning", "notification",
                                                            "information", "debug"]},
                                   "status": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]}
                               }},
                "switch-profile": {"required": False, "type": "str"},
                "switch-stp-settings": {"required": False, "type": "dict",
                                        "options": {
                                            "status": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]}
                                        }},
                "type": {"required": False, "type": "str",
                         "choices": ["virtual", "physical"]},
                "version": {"required": False, "type": "int"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_switch_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2017, James Mighion <@jmighion>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options:
  provider:
    description:
      - A dict object containing connection details.
    suboptions:
      host:
        description:
          - Specifies the DNS host name or address for connecting to the remote
            device over the specified transport.  The value of host is used as
            the destination address for the transport.
        type: str
        required: true
      port:
        description:
          - Specifies the port to use when building the connection to the remote.
            device.
        type: int
        default: 22
      username:
        description:
          - Configures the username to use to authenticate the connection to
            the remote device.  This value is used to authenticate
            the SSH session. If the value is not specified in the task, the
            value of environment variable C(ANSIBLE_NET_USERNAME) will be used instead.
        type: str
      password:
        description:
          - Specifies the password to use to authenticate the connection to
            the remote device.   This value is used to authenticate
            the SSH session. If the value is not specified in the task, the
            value of environment variable C(ANSIBLE_NET_PASSWORD) will be used instead.
        type: str
      timeout:
        description:
          - Specifies the timeout in seconds for communicating with the network device
            for either connecting or sending commands.  If the timeout is
            exceeded before the operation is completed, the module will error.
        type: int
        default: 10
      ssh_keyfile:
        description:
          - Specifies the SSH key to use to authenticate the connection to
            the remote device.   This value is the path to the
            key used to authenticate the SSH session. If the value is not specified
            in the task, the value of environment variable C(ANSIBLE_NET_SSH_KEYFILE)
            will be used instead.
        type: path
'''



#!/usr/bin/python
# coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: get_certificate
author: "John Westcott IV (@john-westcott-iv)"
version_added: "2.8"
short_description: Get a certificate from a host:port
description:
    - Makes a secure connection and returns information about the presented certificate
options:
    host:
      description:
        - The host to get the cert for (IP is fine)
      type: str
      required: true
    ca_cert:
      description:
        - A PEM file containing one or more root certificates; if present, the cert will be validated against these root certs.
        - Note that this only validates the certificate is signed by the chain; not that the cert is valid for the host presenting it.
      type: path
    port:
      description:
        - The port to connect to
      type: int
      required: true
    timeout:
      description:
        - The timeout in seconds
      type: int
      default: 10

notes:
  - When using ca_cert on OS X it has been reported that in some conditions the validate will always succeed.

requirements:
  - "pyOpenSSL >= 0.15"
'''

RETURN = '''
cert:
    description: The certificate retrieved from the port
    returned: success
    type: str
expired:
    description: Boolean indicating if the cert is expired
    returned: success
    type: bool
extensions:
    description: Extensions applied to the cert
    returned: success
    type: list
issuer:
    description: Information about the issuer of the cert
    returned: success
    type: dict
not_after:
    description: Expiration date of the cert
    returned: success
    type: str
not_before:
    description: Issue date of the cert
    returned: success
    type: str
serial_number:
    description: The serial number of the cert
    returned: success
    type: str
signature_algorithm:
    description: The algorithm used to sign the cert
    returned: success
    type: str
subject:
    description: Information about the subject of the cert (OU, CN, etc)
    returned: success
    type: dict
version:
    description: The version number of the certificate
    returned: success
    type: str
'''

EXAMPLES = '''
- name: Get the cert from an RDP port
  get_certificate:
    host: "1.2.3.4"
    port: 3389
  delegate_to: localhost
  run_once: true
  register: cert

- name: Get a cert from an https port
  get_certificate:
    host: "www.google.com"
    port: 443
  delegate_to: localhost
  run_once: true
  register: cert

- name: How many days until cert expires
  debug:
    msg: "cert expires in: {{ expire_days }} days."
  vars:
    expire_days: "{{ (( cert.not_after | to_datetime('%Y%m%d%H%M%SZ')) - (ansible_date_time.iso8601 | to_datetime('%Y-%m-%dT%H:%M:%SZ')) ).days }}"
'''

import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

from os.path import isfile
from ssl import get_server_certificate
from socket import setdefaulttimeout

PYOPENSSL_IMP_ERR = None
try:
    from OpenSSL import crypto
except ImportError:
    PYOPENSSL_IMP_ERR = traceback.format_exc()
    pyopenssl_found = False
else:
    pyopenssl_found = True


def main():
    module = AnsibleModule(
        argument_spec=dict(
            ca_cert=dict(type='path'),
            host=dict(type='str', required=True),
            port=dict(type='int', required=True),
            timeout=dict(type='int', default=10),
        ),
    )

    ca_cert = module.params.get('ca_cert')
    host = module.params.get('host')
    port = module.params.get('port')
    timeout = module.params.get('timeout')

    result = dict(
        changed=False,
    )

    if not pyopenssl_found:
        module.fail_json(msg=missing_required_lib('pyOpenSSL >= 0.15'), exception=PYOPENSSL_IMP_ERR)

    if timeout:
        setdefaulttimeout(timeout)

    if ca_cert:
        if not isfile(ca_cert):
            module.fail_json(msg="ca_cert file does not exist")

    try:
        cert = get_server_certificate((host, port), ca_certs=ca_cert)
        x509 = crypto.load_certificate(crypto.FILETYPE_PEM, cert)
    except Exception as e:
        module.fail_json(msg="Failed to get cert from port with error: {0}".format(e))

    result['cert'] = cert
    result['subject'] = {}
    for component in x509.get_subject().get_components():
        result['subject'][component[0]] = component[1]

    result['expired'] = x509.has_expired()

    result['extensions'] = []
    extension_count = x509.get_extension_count()
    for index in range(0, extension_count):
        extension = x509.get_extension(index)
        result['extensions'].append({
            'critical': extension.get_critical(),
            'asn1_data': extension.get_data(),
            'name': extension.get_short_name(),
        })

    result['issuer'] = {}
    for component in x509.get_issuer().get_components():
        result['issuer'][component[0]] = component[1]

    result['not_after'] = x509.get_notAfter()
    result['not_before'] = x509.get_notBefore()

    result['serial_number'] = x509.get_serial_number()
    result['signature_algorithm'] = x509.get_signature_algorithm()

    result['version'] = x509.get_version()

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_ips_sensor
short_description: Configure IPS sensor.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure ips feature and sensor category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    ips_sensor:
        description:
            - Configure IPS sensor.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            block-malicious-url:
                description:
                    - Enable/disable malicious URL blocking.
                choices:
                    - disable
                    - enable
            comment:
                description:
                    - Comment.
            entries:
                description:
                    - IPS sensor filter.
                suboptions:
                    action:
                        description:
                            - Action taken with traffic in which signatures are detected.
                        choices:
                            - pass
                            - block
                            - reset
                            - default
                    application:
                        description:
                            - Applications to be protected. set application ? lists available applications. all includes all applications. other includes all
                               unlisted applications.
                    exempt-ip:
                        description:
                            - Traffic from selected source or destination IP addresses is exempt from this signature.
                        suboptions:
                            dst-ip:
                                description:
                                    - Destination IP address and netmask.
                            id:
                                description:
                                    - Exempt IP ID.
                                required: true
                            src-ip:
                                description:
                                    - Source IP address and netmask.
                    id:
                        description:
                            - Rule ID in IPS database (0 - 4294967295).
                        required: true
                    location:
                        description:
                            - Protect client or server traffic.
                    log:
                        description:
                            - Enable/disable logging of signatures included in filter.
                        choices:
                            - disable
                            - enable
                    log-attack-context:
                        description:
                            - "Enable/disable logging of attack context: URL buffer, header buffer, body buffer, packet buffer."
                        choices:
                            - disable
                            - enable
                    log-packet:
                        description:
                            - Enable/disable packet logging. Enable to save the packet that triggers the filter. You can download the packets in pcap format
                               for diagnostic use.
                        choices:
                            - disable
                            - enable
                    os:
                        description:
                            - Operating systems to be protected.  all includes all operating systems. other includes all unlisted operating systems.
                    protocol:
                        description:
                            - Protocols to be examined. set protocol ? lists available protocols. all includes all protocols. other includes all unlisted
                               protocols.
                    quarantine:
                        description:
                            - Quarantine method.
                        choices:
                            - none
                            - attacker
                    quarantine-expiry:
                        description:
                            - Duration of quarantine. (Format ###d##h##m, minimum 1m, maximum 364d23h59m, default = 5m). Requires quarantine set to attacker.
                    quarantine-log:
                        description:
                            - Enable/disable quarantine logging.
                        choices:
                            - disable
                            - enable
                    rate-count:
                        description:
                            - Count of the rate.
                    rate-duration:
                        description:
                            - Duration (sec) of the rate.
                    rate-mode:
                        description:
                            - Rate limit mode.
                        choices:
                            - periodical
                            - continuous
                    rate-track:
                        description:
                            - Track the packet protocol field.
                        choices:
                            - none
                            - src-ip
                            - dest-ip
                            - dhcp-client-mac
                            - dns-domain
                    rule:
                        description:
                            - Identifies the predefined or custom IPS signatures to add to the sensor.
                        suboptions:
                            id:
                                description:
                                    - Rule IPS.
                                required: true
                    severity:
                        description:
                            - Relative severity of the signature, from info to critical. Log messages generated by the signature include the severity.
                    status:
                        description:
                            - Status of the signatures included in filter. default enables the filter and only use filters with default status of enable.
                               Filters with default status of disable will not be used.
                        choices:
                            - disable
                            - enable
                            - default
            extended-log:
                description:
                    - Enable/disable extended logging.
                choices:
                    - enable
                    - disable
            filter:
                description:
                    - IPS sensor filter.
                suboptions:
                    action:
                        description:
                            - Action of selected rules.
                        choices:
                            - pass
                            - block
                            - reset
                            - default
                    application:
                        description:
                            - Vulnerable application filter.
                    location:
                        description:
                            - Vulnerability location filter.
                    log:
                        description:
                            - Enable/disable logging of selected rules.
                        choices:
                            - disable
                            - enable
                    log-packet:
                        description:
                            - Enable/disable packet logging of selected rules.
                        choices:
                            - disable
                            - enable
                    name:
                        description:
                            - Filter name.
                        required: true
                    os:
                        description:
                            - Vulnerable OS filter.
                    protocol:
                        description:
                            - Vulnerable protocol filter.
                    quarantine:
                        description:
                            - Quarantine IP or interface.
                        choices:
                            - none
                            - attacker
                    quarantine-expiry:
                        description:
                            - Duration of quarantine in minute.
                    quarantine-log:
                        description:
                            - Enable/disable logging of selected quarantine.
                        choices:
                            - disable
                            - enable
                    severity:
                        description:
                            - Vulnerability severity filter.
                    status:
                        description:
                            - Selected rules status.
                        choices:
                            - disable
                            - enable
                            - default
            name:
                description:
                    - Sensor name.
                required: true
            override:
                description:
                    - IPS override rule.
                suboptions:
                    action:
                        description:
                            - Action of override rule.
                        choices:
                            - pass
                            - block
                            - reset
                    exempt-ip:
                        description:
                            - Exempted IP.
                        suboptions:
                            dst-ip:
                                description:
                                    - Destination IP address and netmask.
                            id:
                                description:
                                    - Exempt IP ID.
                                required: true
                            src-ip:
                                description:
                                    - Source IP address and netmask.
                    log:
                        description:
                            - Enable/disable logging.
                        choices:
                            - disable
                            - enable
                    log-packet:
                        description:
                            - Enable/disable packet logging.
                        choices:
                            - disable
                            - enable
                    quarantine:
                        description:
                            - Quarantine IP or interface.
                        choices:
                            - none
                            - attacker
                    quarantine-expiry:
                        description:
                            - Duration of quarantine in minute.
                    quarantine-log:
                        description:
                            - Enable/disable logging of selected quarantine.
                        choices:
                            - disable
                            - enable
                    rule-id:
                        description:
                            - Override rule ID.
                        required: true
                    status:
                        description:
                            - Enable/disable status of override rule.
                        choices:
                            - disable
                            - enable
            replacemsg-group:
                description:
                    - Replacement message group. Source system.replacemsg-group.name.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPS sensor.
    fortios_ips_sensor:
      host:  "{{  host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{  vdom }}"
      ips_sensor:
        state: "present"
        block-malicious-url: "disable"
        comment: "Comment."
        entries:
         -
            action: "pass"
            application: "<your_own_value>"
            exempt-ip:
             -
                dst-ip: "<your_own_value>"
                id:  "10"
                src-ip: "<your_own_value>"
            id:  "12"
            location: "<your_own_value>"
            log: "disable"
            log-attack-context: "disable"
            log-packet: "disable"
            os: "<your_own_value>"
            protocol: "<your_own_value>"
            quarantine: "none"
            quarantine-expiry: "<your_own_value>"
            quarantine-log: "disable"
            rate-count: "22"
            rate-duration: "23"
            rate-mode: "periodical"
            rate-track: "none"
            rule:
             -
                id:  "27"
            severity: "<your_own_value>"
            status: "disable"
        extended-log: "enable"
        filter:
         -
            action: "pass"
            application: "<your_own_value>"
            location: "<your_own_value>"
            log: "disable"
            log-packet: "disable"
            name: "default_name_37"
            os: "<your_own_value>"
            protocol: "<your_own_value>"
            quarantine: "none"
            quarantine-expiry: "41"
            quarantine-log: "disable"
            severity: "<your_own_value>"
            status: "disable"
        name: "default_name_45"
        override:
         -
            action: "pass"
            exempt-ip:
             -
                dst-ip: "<your_own_value>"
                id:  "50"
                src-ip: "<your_own_value>"
            log: "disable"
            log-packet: "disable"
            quarantine: "none"
            quarantine-expiry: "55"
            quarantine-log: "disable"
            rule-id: "57"
            status: "disable"
        replacemsg-group: "<your_own_value> (source system.replacemsg-group.name)"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_ips_sensor_data(json):
    option_list = ['block-malicious-url', 'comment', 'entries',
                   'extended-log', 'filter', 'name',
                   'override', 'replacemsg-group']
    dictionary = {}

    for attribute in option_list:
        if attribute in json:
            dictionary[attribute] = json[attribute]

    return dictionary


def ips_sensor(data, fos):
    vdom = data['vdom']
    ips_sensor_data = data['ips_sensor']
    filtered_data = filter_ips_sensor_data(ips_sensor_data)
    if ips_sensor_data['state'] == "present":
        return fos.set('ips',
                       'sensor',
                       data=filtered_data,
                       vdom=vdom)

    elif ips_sensor_data['state'] == "absent":
        return fos.delete('ips',
                          'sensor',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_ips(data, fos):
    login(data)

    methodlist = ['ips_sensor']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "ips_sensor": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "block-malicious-url": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                "comment": {"required": False, "type": "str"},
                "entries": {"required": False, "type": "list",
                            "options": {
                                "action": {"required": False, "type": "str",
                                           "choices": ["pass", "block", "reset",
                                                       "default"]},
                                "application": {"required": False, "type": "str"},
                                "exempt-ip": {"required": False, "type": "list",
                                              "options": {
                                                  "dst-ip": {"required": False, "type": "str"},
                                                  "id": {"required": True, "type": "int"},
                                                  "src-ip": {"required": False, "type": "str"}
                                              }},
                                "id": {"required": True, "type": "int"},
                                "location": {"required": False, "type": "str"},
                                "log": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                                "log-attack-context": {"required": False, "type": "str",
                                                       "choices": ["disable", "enable"]},
                                "log-packet": {"required": False, "type": "str",
                                               "choices": ["disable", "enable"]},
                                "os": {"required": False, "type": "str"},
                                "protocol": {"required": False, "type": "str"},
                                "quarantine": {"required": False, "type": "str",
                                               "choices": ["none", "attacker"]},
                                "quarantine-expiry": {"required": False, "type": "str"},
                                "quarantine-log": {"required": False, "type": "str",
                                                   "choices": ["disable", "enable"]},
                                "rate-count": {"required": False, "type": "int"},
                                "rate-duration": {"required": False, "type": "int"},
                                "rate-mode": {"required": False, "type": "str",
                                              "choices": ["periodical", "continuous"]},
                                "rate-track": {"required": False, "type": "str",
                                               "choices": ["none", "src-ip", "dest-ip",
                                                           "dhcp-client-mac", "dns-domain"]},
                                "rule": {"required": False, "type": "list",
                                         "options": {
                                             "id": {"required": True, "type": "int"}
                                         }},
                                "severity": {"required": False, "type": "str"},
                                "status": {"required": False, "type": "str",
                                           "choices": ["disable", "enable", "default"]}
                            }},
                "extended-log": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "filter": {"required": False, "type": "list",
                           "options": {
                               "action": {"required": False, "type": "str",
                                          "choices": ["pass", "block", "reset",
                                                      "default"]},
                               "application": {"required": False, "type": "str"},
                               "location": {"required": False, "type": "str"},
                               "log": {"required": False, "type": "str",
                                       "choices": ["disable", "enable"]},
                               "log-packet": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                               "name": {"required": True, "type": "str"},
                               "os": {"required": False, "type": "str"},
                               "protocol": {"required": False, "type": "str"},
                               "quarantine": {"required": False, "type": "str",
                                              "choices": ["none", "attacker"]},
                               "quarantine-expiry": {"required": False, "type": "int"},
                               "quarantine-log": {"required": False, "type": "str",
                                                  "choices": ["disable", "enable"]},
                               "severity": {"required": False, "type": "str"},
                               "status": {"required": False, "type": "str",
                                          "choices": ["disable", "enable", "default"]}
                           }},
                "name": {"required": True, "type": "str"},
                "override": {"required": False, "type": "list",
                             "options": {
                                 "action": {"required": False, "type": "str",
                                            "choices": ["pass", "block", "reset"]},
                                 "exempt-ip": {"required": False, "type": "list",
                                               "options": {
                                                   "dst-ip": {"required": False, "type": "str"},
                                                   "id": {"required": True, "type": "int"},
                                                   "src-ip": {"required": False, "type": "str"}
                                               }},
                                 "log": {"required": False, "type": "str",
                                         "choices": ["disable", "enable"]},
                                 "log-packet": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                                 "quarantine": {"required": False, "type": "str",
                                                "choices": ["none", "attacker"]},
                                 "quarantine-expiry": {"required": False, "type": "int"},
                                 "quarantine-log": {"required": False, "type": "str",
                                                    "choices": ["disable", "enable"]},
                                 "rule-id": {"required": True, "type": "int"},
                                 "status": {"required": False, "type": "str",
                                            "choices": ["disable", "enable"]}
                             }},
                "replacemsg-group": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_ips(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wireless_controller_global
short_description: Configure wireless controller global settings in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify wireless_controller feature and global category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    wireless_controller_global:
        description:
            - Configure wireless controller global settings.
        default: null
        suboptions:
            ap-log-server:
                description:
                    - Enable/disable configuring APs or FortiAPs to send log messages to a syslog server (default = disable).
                choices:
                    - enable
                    - disable
            ap-log-server-ip:
                description:
                    - IP address that APs or FortiAPs send log messages to.
            ap-log-server-port:
                description:
                    - Port that APs or FortiAPs send log messages to.
            control-message-offload:
                description:
                    - Configure CAPWAP control message data channel offload.
                choices:
                    - ebp-frame
                    - aeroscout-tag
                    - ap-list
                    - sta-list
                    - sta-cap-list
                    - stats
                    - aeroscout-mu
            data-ethernet-II:
                description:
                    - Configure the wireless controller to use Ethernet II or 802.3 frames with 802.3 data tunnel mode (default = disable).
                choices:
                    - enable
                    - disable
            discovery-mc-addr:
                description:
                    - Multicast IP address for AP discovery (default = 244.0.1.140).
            fiapp-eth-type:
                description:
                    - Ethernet type for Fortinet Inter-Access Point Protocol (IAPP), or IEEE 802.11f, packets (0 - 65535, default = 5252).
            image-download:
                description:
                    - Enable/disable WTP image download at join time.
                choices:
                    - enable
                    - disable
            ipsec-base-ip:
                description:
                    - Base IP address for IPsec VPN tunnels between the access points and the wireless controller (default = 169.254.0.1).
            link-aggregation:
                description:
                    - Enable/disable calculating the CAPWAP transmit hash to load balance sessions to link aggregation nodes (default = disable).
                choices:
                    - enable
                    - disable
            location:
                description:
                    - Description of the location of the wireless controller.
            max-clients:
                description:
                    - Maximum number of clients that can connect simultaneously (default = 0, meaning no limitation).
            max-retransmit:
                description:
                    - Maximum number of tunnel packet retransmissions (0 - 64, default = 3).
            mesh-eth-type:
                description:
                    - Mesh Ethernet identifier included in backhaul packets (0 - 65535, default = 8755).
            name:
                description:
                    - Name of the wireless controller.
            rogue-scan-mac-adjacency:
                description:
                    - Maximum numerical difference between an AP's Ethernet and wireless MAC values to match for rogue detection (0 - 31, default = 7).
            wtp-share:
                description:
                    - Enable/disable sharing of WTPs between VDOMs.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure wireless controller global settings.
    fortios_wireless_controller_global:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      wireless_controller_global:
        ap-log-server: "enable"
        ap-log-server-ip: "<your_own_value>"
        ap-log-server-port: "5"
        control-message-offload: "ebp-frame"
        data-ethernet-II: "enable"
        discovery-mc-addr: "<your_own_value>"
        fiapp-eth-type: "9"
        image-download: "enable"
        ipsec-base-ip: "<your_own_value>"
        link-aggregation: "enable"
        location: "<your_own_value>"
        max-clients: "14"
        max-retransmit: "15"
        mesh-eth-type: "16"
        name: "default_name_17"
        rogue-scan-mac-adjacency: "18"
        wtp-share: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_wireless_controller_global_data(json):
    option_list = ['ap-log-server', 'ap-log-server-ip', 'ap-log-server-port',
                   'control-message-offload', 'data-ethernet-II', 'discovery-mc-addr',
                   'fiapp-eth-type', 'image-download', 'ipsec-base-ip',
                   'link-aggregation', 'location', 'max-clients',
                   'max-retransmit', 'mesh-eth-type', 'name',
                   'rogue-scan-mac-adjacency', 'wtp-share']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def wireless_controller_global(data, fos):
    vdom = data['vdom']
    wireless_controller_global_data = data['wireless_controller_global']
    filtered_data = filter_wireless_controller_global_data(wireless_controller_global_data)

    return fos.set('wireless-controller',
                   'global',
                   data=filtered_data,
                   vdom=vdom)


def fortios_wireless_controller(data, fos):
    login(data, fos)

    if data['wireless_controller_global']:
        resp = wireless_controller_global(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "wireless_controller_global": {
            "required": False, "type": "dict",
            "options": {
                "ap-log-server": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "ap-log-server-ip": {"required": False, "type": "str"},
                "ap-log-server-port": {"required": False, "type": "int"},
                "control-message-offload": {"required": False, "type": "str",
                                            "choices": ["ebp-frame", "aeroscout-tag", "ap-list",
                                                        "sta-list", "sta-cap-list", "stats",
                                                        "aeroscout-mu"]},
                "data-ethernet-II": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "discovery-mc-addr": {"required": False, "type": "ipv4-address-multicast"},
                "fiapp-eth-type": {"required": False, "type": "int"},
                "image-download": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "ipsec-base-ip": {"required": False, "type": "str"},
                "link-aggregation": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "location": {"required": False, "type": "str"},
                "max-clients": {"required": False, "type": "int"},
                "max-retransmit": {"required": False, "type": "int"},
                "mesh-eth-type": {"required": False, "type": "int"},
                "name": {"required": False, "type": "str"},
                "rogue-scan-mac-adjacency": {"required": False, "type": "int"},
                "wtp-share": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_wireless_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#
# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json
import re
import time

from ansible.errors import AnsibleConnectionFailure
from ansible.plugins.terminal import TerminalBase


class TerminalModule(TerminalBase):

    terminal_stdout_re = [
        re.compile(br"[\r\n]?[\w]*\(.+\)?[>#\$](?:\s*)$"),
        re.compile(br"User:")
    ]

    terminal_stderr_re = [
        re.compile(br"% ?Error"),
        re.compile(br"% ?Bad secret"),
        re.compile(br"invalid input", re.I),
        re.compile(br"incorrect usage", re.I),
        re.compile(br"(?:incomplete|ambiguous) command", re.I),
        re.compile(br"connection timed out", re.I),
        re.compile(br"[^\r\n]+ not found", re.I),
        re.compile(br"'[^']' +returned error code: ?\d+"),
    ]

    def on_open_shell(self):
        try:
            commands = ('{"command": "' + self._connection._play_context.remote_user + '", "prompt": "Password:", "answer": "' +
                        self._connection._play_context.password + '"}',
                        '{"command": "config paging disable"}')
            for cmd in commands:
                self._exec_cli_command(cmd)
        except AnsibleConnectionFailure:
            try:
                self._exec_cli_command(b'config paging disable')
            except AnsibleConnectionFailure:
                raise AnsibleConnectionFailure('unable to set terminal parameters')



# Copyright: (c) 2012, Jan-Piet Mens <jpmens () gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible import constants as C
from ansible.release import __version__ as ansible_version
from ansible.errors import AnsibleError, AnsibleAssertionError
from ansible.module_utils.six import string_types
from ansible.module_utils._text import to_native
from ansible.module_utils.common._collections_compat import MutableMapping, MutableSet, MutableSequence
from ansible.parsing.plugin_docs import read_docstring
from ansible.parsing.yaml.loader import AnsibleLoader
from ansible.utils.display import Display

display = Display()


# modules that are ok that they do not have documentation strings
BLACKLIST = {
    'MODULE': frozenset(('async_wrapper',)),
    'CACHE': frozenset(('base',)),
}


def merge_fragment(target, source):

    for key, value in source.items():
        if key in target:
            # assumes both structures have same type
            if isinstance(target[key], MutableMapping):
                value.update(target[key])
            elif isinstance(target[key], MutableSet):
                value.add(target[key])
            elif isinstance(target[key], MutableSequence):
                value = sorted(frozenset(value + target[key]))
            else:
                raise Exception("Attempt to extend a documentation fragement, invalid type for %s" % key)
        target[key] = value


def add_fragments(doc, filename, fragment_loader):

    fragments = doc.pop('extends_documentation_fragment', [])

    if isinstance(fragments, string_types):
        fragments = [fragments]

    # Allow the module to specify a var other than DOCUMENTATION
    # to pull the fragment from, using dot notation as a separator
    for fragment_slug in fragments:
        fallback_name = None
        fragment_slug = fragment_slug.lower()
        if '.' in fragment_slug:
            fallback_name = fragment_slug
            fragment_name, fragment_var = fragment_slug.rsplit('.', 1)
            fragment_var = fragment_var.upper()
        else:
            fragment_name, fragment_var = fragment_slug, 'DOCUMENTATION'

        fragment_class = fragment_loader.get(fragment_name)
        if fragment_class is None and fallback_name:
            fragment_class = fragment_loader.get(fallback_name)
            fragment_var = 'DOCUMENTATION'

        if fragment_class is None:
            raise AnsibleAssertionError('fragment_class is None')

        fragment_yaml = getattr(fragment_class, fragment_var, '{}')
        fragment = AnsibleLoader(fragment_yaml, file_name=filename).get_single_data()

        if 'notes' in fragment:
            notes = fragment.pop('notes')
            if notes:
                if 'notes' not in doc:
                    doc['notes'] = []
                doc['notes'].extend(notes)

        if 'seealso' in fragment:
            seealso = fragment.pop('seealso')
            if seealso:
                if 'seealso' not in doc:
                    doc['seealso'] = []
                doc['seealso'].extend(seealso)

        if 'options' not in fragment:
            raise Exception("missing options in fragment (%s), possibly misformatted?: %s" % (fragment_name, filename))

        # ensure options themselves are directly merged
        if 'options' in doc:
            try:
                merge_fragment(doc['options'], fragment.pop('options'))
            except Exception as e:
                raise AnsibleError("%s options (%s) of unknown type: %s" % (to_native(e), fragment_name, filename))
        else:
            doc['options'] = fragment.pop('options')

        # merge rest of the sections
        try:
            merge_fragment(doc, fragment)
        except Exception as e:
            raise AnsibleError("%s (%s) of unknown type: %s" % (to_native(e), fragment_name, filename))


def get_docstring(filename, fragment_loader, verbose=False, ignore_errors=False):
    """
    DOCUMENTATION can be extended using documentation fragments loaded by the PluginLoader from the doc_fragments plugins.
    """

    data = read_docstring(filename, verbose=verbose, ignore_errors=ignore_errors)

    # add fragments to documentation
    if data.get('doc', False):
        add_fragments(data['doc'], filename, fragment_loader=fragment_loader)

    return data['doc'], data['plainexamples'], data['returndocs'], data['metadata']


def get_versioned_doclink(path):
    """
    returns a versioned documentation link for the current Ansible major.minor version; used to generate
    in-product warning/error links to the configured DOCSITE_ROOT_URL
    (eg, https://docs.ansible.com/ansible/2.8/somepath/doc.html)

    :param path: relative path to a document under docs/docsite/rst;
    :return: absolute URL to the specified doc for the current version of Ansible
    """
    path = to_native(path)
    try:
        base_url = C.config.get_config_value('DOCSITE_ROOT_URL')
        if not base_url.endswith('/'):
            base_url += '/'
        if path.startswith('/'):
            path = path[1:]
        split_ver = ansible_version.split('.')
        if len(split_ver) < 3:
            raise RuntimeError('invalid version ({0})'.format(ansible_version))

        doc_version = '{0}.{1}'.format(split_ver[0], split_ver[1])

        # check to see if it's a X.Y.0 non-rc prerelease or dev release, if so, assume devel (since the X.Y doctree
        # isn't published until beta-ish)
        if split_ver[2].startswith('0'):
            # exclude rc; we should have the X.Y doctree live by rc1
            if any((pre in split_ver[2]) for pre in ['a', 'b']) or len(split_ver) > 3 and 'dev' in split_ver[3]:
                doc_version = 'devel'

        return '{0}{1}/{2}'.format(base_url, doc_version, path)
    except Exception as ex:
        return '(unable to create versioned doc link for path {0}: {1})'.format(path, to_native(ex))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Hetzner Cloud GmbH <info@hetzner-cloud.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: hcloud_ssh_key

short_description: Create and manage ssh keys on the Hetzner Cloud.

version_added: "2.8"

description:
    - Create, update and manage ssh keys on the Hetzner Cloud.

author:
    - Lukas Kaemmerling (@lkaemmerling)

options:
    id:
        description:
            - The ID of the Hetzner Cloud ssh_key to manage.
            - Only required if no ssh_key I(name) is given
        type: int
    name:
        description:
            - The Name of the Hetzner Cloud ssh_key to manage.
            - Only required if no ssh_key I(id) is given or a ssh_key does not exists.
        type: str
    fingerprint:
        description:
            - The Fingerprint of the Hetzner Cloud ssh_key to manage.
            - Only required if no ssh_key I(id) or I(name) is given.
        type: str
    labels:
        description:
            - User-defined labels (key-value pairs)
        type: dict
    public_key:
        description:
            - The Public Key to add.
            - Required if ssh_key does not exists.
        type: str
    state:
        description:
            - State of the ssh_key.
        default: present
        choices: [ absent, present ]
        type: str
extends_documentation_fragment: hcloud
"""

EXAMPLES = """
- name: Create a basic ssh_key
  hcloud_ssh_key:
    name: my-ssh_key
    public_key: "ssh-rsa AAAjjk76kgf...Xt"
    state: present

- name: Create a ssh_key with labels
  hcloud_ssh_key:
    name: my-ssh_key
    public_key: "ssh-rsa AAAjjk76kgf...Xt"
    labels:
        key: value
        mylabel: 123
    state: present

- name: Ensure the ssh_key is absent (remove if needed)
  hcloud_ssh_key:
    name: my-ssh_key
    state: absent
"""

RETURN = """
hcloud_ssh_key:
    description: The ssh_key instance
    returned: Always
    type: complex
    contains:
        id:
            description: ID of the ssh_key
            type: int
            returned: Always
            sample: 12345
        name:
            description: Name of the ssh_key
            type: string
            returned: Always
            sample: my-ssh-key
        fingerprint:
            description: Fingerprint of the ssh_key
            type: string
            returned: Always
            sample: b7:2f:30:a0:2f:6c:58:6c:21:04:58:61:ba:06:3b:2f
        public_key:
            description: Public key of the ssh_key
            type: string
            returned: Always
            sample: "ssh-rsa AAAjjk76kgf...Xt"
        labels:
            description: User-defined labels (key-value pairs)
            type: dict
            returned: Always
            sample:
                key: value
                mylabel: 123
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
from ansible.module_utils.hcloud import Hcloud

try:
    from hcloud.volumes.domain import Volume
    from hcloud.ssh_keys.domain import SSHKey
    from hcloud.ssh_keys.domain import Server
    from hcloud import APIException
except ImportError:
    pass


class AnsibleHcloudSSHKey(Hcloud):
    def __init__(self, module):
        Hcloud.__init__(self, module, "hcloud_ssh_key")
        self.hcloud_ssh_key = None

    def _prepare_result(self):
        return {
            "id": to_native(self.hcloud_ssh_key.id),
            "name": to_native(self.hcloud_ssh_key.name),
            "fingerprint": to_native(self.hcloud_ssh_key.fingerprint),
            "public_key": to_native(self.hcloud_ssh_key.public_key),
            "labels": self.hcloud_ssh_key.labels,
        }

    def _get_ssh_key(self):
        try:
            if self.module.params.get("id") is not None:
                self.hcloud_ssh_key = self.client.ssh_keys.get_by_id(
                    self.module.params.get("id")
                )
            elif self.module.params.get("fingerprint") is not None:
                self.hcloud_ssh_key = self.client.ssh_keys.get_by_fingerprint(
                    self.module.params.get("fingerprint")
                )
            elif self.module.params.get("name") is not None:
                self.hcloud_ssh_key = self.client.ssh_keys.get_by_name(
                    self.module.params.get("name")
                )

        except APIException as e:
            self.module.fail_json(msg=e.message)

    def _create_ssh_key(self):
        self.module.fail_on_missing_params(
            required_params=["name", "public_key"]
        )
        params = {
            "name": self.module.params.get("name"),
            "public_key": self.module.params.get("public_key"),
            "labels": self.module.params.get("labels")
        }

        if not self.module.check_mode:
            self.client.ssh_keys.create(**params)
        self._mark_as_changed()
        self._get_ssh_key()

    def _update_ssh_key(self):
        name = self.module.params.get("name")
        if name is not None and self.hcloud_ssh_key.name != name:
            self.module.fail_on_missing_params(
                required_params=["id"]
            )
            if not self.module.check_mode:
                self.hcloud_ssh_key.update(name=name)
            self._mark_as_changed()

        labels = self.module.params.get("labels")
        if labels is not None and self.hcloud_ssh_key.labels != labels:
            if not self.module.check_mode:
                self.hcloud_ssh_key.update(labels=labels)
            self._mark_as_changed()

        self._get_ssh_key()

    def present_ssh_key(self):
        self._get_ssh_key()
        if self.hcloud_ssh_key is None:
            self._create_ssh_key()
        else:
            self._update_ssh_key()

    def delete_ssh_key(self):
        self._get_ssh_key()
        if self.hcloud_ssh_key is not None:
            if not self.module.check_mode:
                self.client.ssh_keys.delete(self.hcloud_ssh_key)
            self._mark_as_changed()
        self.hcloud_ssh_key = None

    @staticmethod
    def define_module():
        return AnsibleModule(
            argument_spec=dict(
                id={"type": "int"},
                name={"type": "str"},
                public_key={"type": "str"},
                fingerprint={"type": "str"},
                labels={"type": "dict"},
                state={
                    "choices": ["absent", "present"],
                    "default": "present",
                },
                **Hcloud.base_module_arguments()
            ),
            required_one_of=[['id', 'name', 'fingerprint']],
            required_if=[['state', 'present', ['name']]],
            supports_check_mode=True,
        )


def main():
    module = AnsibleHcloudSSHKey.define_module()

    hcloud = AnsibleHcloudSSHKey(module)
    state = module.params.get("state")
    if state == "absent":
        hcloud.delete_ssh_key()
    elif state == "present":
        hcloud.present_ssh_key()

    module.exit_json(**hcloud.get_result())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}

DOCUMENTATION = """
---
module: skydive_capture
version_added: "2.8"
author:
  - "Sumit Jaiswal (@sjaiswal)"
short_description: Module which manages flow capture on interfaces
description:
  - This module manages flow capture on interfaces. The Gremlin
    expression is continuously evaluated which means that it is
    possible to define a capture on nodes that do not exist yet.
  - It is useful when you want to start a capture on all OpenvSwitch
    whatever the number of Skydive agents you will start.
  - While starting the capture, user can specify the capture name,
    capture description and capture type optionally.
requirements:
  - skydive-client
extends_documentation_fragment: skydive
options:
  query:
    description:
      - It's the complete gremlin query which the users can input,
        I(G.V().Has('Name', 'eth0', 'Type', 'device')), to create
        the capture. And, if the user directly inputs the gremlin
        query then user is not required to input any other module
        parameter as gremlin query takes care of creating the flow
        capture.
    required: false
  interface_name:
    description:
      - To define flow capture interface name.
    required: false
  type:
    description:
      - To define flow capture interface type.
    required: false
  capture_name:
    description:
      - To define flow capture name.
    required: false
    default: ""
  description:
    description:
      - Configures a text string to be associated with the instance
        of this object.
    default: ""
  extra_tcp_metric:
    description:
      - To define flow capture ExtraTCPMetric.
    type: bool
    default: false
  ip_defrag:
    description:
      - To define flow capture IPDefrag.
    type: bool
    default: false
  reassemble_tcp:
    description:
      - To define flow capture ReassembleTCP.
    type: bool
    default: false
  layer_key_mode:
    description:
      - To define flow capture Layer KeyMode.
    type: str
    default: L2
  state:
    description:
      - State of the flow capture. If value is I(present) flow capture
        will be created else if it is I(absent) it will be deleted.
    default: present
    choices:
      - present
      - absent
"""

EXAMPLES = """
- name: start a new flow capture directly from gremlin query
  skydive_capture:
    query: G.V().Has('Name', 'eth0', 'Type', 'device')
    state: present
    provider:
      endpoint: localhost:8082
      username: admin
      password: admin

- name: stop the flow capture directly from gremlin query
  skydive_capture:
    query: G.V().Has('Name', 'eth0', 'Type', 'device')
    state: absent
    provider:
      endpoint: localhost:8082
      username: admin
      password: admin

- name: start a new flow capture from user's input
  skydive_capture:
    interface_name: Node1
    type: myhost
    capture_name: test_capture
    description: test description
    extra_tcp_metric: true
    ip_defrag: true
    reassemble_tcp: true
    state: present
    provider:
      endpoint: localhost:8082
      username: admin
      password: admin

- name: stop the flow capture
  skydive_capture:
    interface_name: Node1
    type: myhost
    capture_name: test_capture
    description: test description
    extra_tcp_metric: true
    ip_defrag: true
    reassemble_tcp: true
    state: absent
    provider:
      endpoint: localhost:8082
      username: admin
      password: admin
"""

RETURN = """ # """

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.skydive.api import skydive_flow_capture


def main():
    ''' Main entry point for module execution
    '''
    ib_spec = dict(
        query=dict(required=False, ib_req=True),
        interface_name=dict(required=False, ib_req=True),
        type=dict(required=False, ib_req=True),
        capture_name=dict(required=False, default='', ib_req=True),
        description=dict(default='', ib_req=True),
        extra_tcp_metric=dict(type='bool', required=False, ib_req=True, default=False),
        ip_defrag=dict(type='bool', required=False, ib_req=True, default=False),
        reassemble_tcp=dict(type='bool', required=False, ib_req=True, default=False),
        layer_key_mode=dict(required=False, ib_req=True, default='L2')
    )

    argument_spec = dict(
        provider=dict(required=False),
        state=dict(default='present', choices=['present', 'absent'])
    )

    argument_spec.update(ib_spec)
    argument_spec.update(skydive_flow_capture.provider_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    skydive_obj = skydive_flow_capture(module)
    result = skydive_obj.run(ib_spec)

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Mikhail Yohman (@FragmentedPacket) <mikhail.yohman@gmail.com>
# Copyright: (c) 2018, David Gomez (@amb1s1) <david.gomez@networktocode.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: netbox_device
short_description: Create, update or delete devices within Netbox
description:
  - Creates, updates or removes devices from Netbox
notes:
  - Tags should be defined as a YAML list
  - This should be ran with connection C(local) and hosts C(localhost)
author:
  - Mikhail Yohman (@FragmentedPacket)
  - David Gomez (@amb1s1)
requirements:
  - pynetbox
version_added: '2.8'
options:
  netbox_url:
    description:
      - URL of the Netbox instance resolvable by Ansible control host
    required: true
  netbox_token:
    description:
      - The token created within Netbox to authorize API access
    required: true
  data:
    description:
      - Defines the device configuration
    suboptions:
      name:
        description:
          - The name of the device
        required: true
      device_type:
        description:
          - Required if I(state=present) and the device does not exist yet
      device_role:
        description:
          - Required if I(state=present) and the device does not exist yet
      tenant:
        description:
          - The tenant that the device will be assigned to
      platform:
        description:
          - The platform of the device
      serial:
        description:
          - Serial number of the device
      asset_tag:
        description:
          - Asset tag that is associated to the device
      site:
        description:
          - Required if I(state=present) and the device does not exist yet
      rack:
        description:
          - The name of the rack to assign the device to
      position:
        description:
          - The position of the device in the rack defined above
      face:
        description:
          - Required if I(rack) is defined
      status:
        description:
          - The status of the device
        choices:
          - Active
          - Offline
          - Planned
          - Staged
          - Failed
          - Inventory
      cluster:
        description:
          - Cluster that the device will be assigned to
      comments:
        description:
          - Comments that may include additional information in regards to the device
      tags:
        description:
          - Any tags that the device may need to be associated with
      custom_fields:
        description:
          - must exist in Netbox
    required: true
  state:
    description:
      - Use C(present) or C(absent) for adding or removing.
    choices: [ absent, present ]
    default: present
  validate_certs:
    description:
      - If C(no), SSL certificates will not be validated. This should only be used on personally controlled sites using self-signed certificates.
    default: 'yes'
    type: bool
'''

EXAMPLES = r'''
- name: "Test Netbox modules"
  connection: local
  hosts: localhost
  gather_facts: False

  tasks:
    - name: Create device within Netbox with only required information
      netbox_device:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          name: Test Device
          device_type: C9410R
          device_role: Core Switch
          site: Main
        state: present

    - name: Delete device within netbox
      netbox_device:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          name: Test Device
        state: absent

    - name: Create device with tags
      netbox_device:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          name: Another Test Device
          device_type: C9410R
          device_role: Core Switch
          site: Main
          tags:
            - Schnozzberry
        state: present

    - name: Update the rack and position of an existing device
      netbox_device:
        netbox_url: http://netbox.local
        netbox_token: thisIsMyToken
        data:
          name: Test Device
          rack: Test Rack
          position: 10
          face: Front
        state: present
'''

RETURN = r'''
device:
  description: Serialized object as created or already existent within Netbox
  returned: success (when I(state=present))
  type: dict
msg:
  description: Message indicating failure or info about what has been achieved
  returned: always
  type: str
'''

import json
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.net_tools.netbox.netbox_utils import (
    find_ids,
    normalize_data,
    create_netbox_object,
    delete_netbox_object,
    update_netbox_object,
    DEVICE_STATUS,
    FACE_ID
)

PYNETBOX_IMP_ERR = None
try:
    import pynetbox
    HAS_PYNETBOX = True
except ImportError:
    PYNETBOX_IMP_ERR = traceback.format_exc()
    HAS_PYNETBOX = False


def main():
    '''
    Main entry point for module execution
    '''
    argument_spec = dict(
        netbox_url=dict(type="str", required=True),
        netbox_token=dict(type="str", required=True, no_log=True),
        data=dict(type="dict", required=True),
        state=dict(required=False, default='present', choices=['present', 'absent']),
        validate_certs=dict(type="bool", default=True)
    )

    global module
    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    # Fail module if pynetbox is not installed
    if not HAS_PYNETBOX:
        module.fail_json(msg=missing_required_lib('pynetbox'), exception=PYNETBOX_IMP_ERR)

    # Fail if device name is not given
    if not module.params["data"].get("name"):
        module.fail_json(msg="missing device name")

    # Assign variables to be used with module
    app = 'dcim'
    endpoint = 'devices'
    url = module.params["netbox_url"]
    token = module.params["netbox_token"]
    data = module.params["data"]
    state = module.params["state"]
    validate_certs = module.params["validate_certs"]

    # Attempt to create Netbox API object
    try:
        nb = pynetbox.api(url, token=token, ssl_verify=validate_certs)
    except Exception:
        module.fail_json(msg="Failed to establish connection to Netbox API")
    try:
        nb_app = getattr(nb, app)
    except AttributeError:
        module.fail_json(msg="Incorrect application specified: %s" % (app))

    nb_endpoint = getattr(nb_app, endpoint)
    norm_data = normalize_data(data)
    try:
        if 'present' in state:
            result = ensure_device_present(nb, nb_endpoint, norm_data)
        else:
            result = ensure_device_absent(nb_endpoint, norm_data)
        return module.exit_json(**result)
    except pynetbox.RequestError as e:
        return module.fail_json(msg=json.loads(e.error))
    except ValueError as e:
        return module.fail_json(msg=str(e))


def _find_ids(nb, data):
    if data.get("status"):
        data["status"] = DEVICE_STATUS.get(data["status"].lower())
    if data.get("face"):
        data["face"] = FACE_ID.get(data["face"].lower())
    return find_ids(nb, data)


def ensure_device_present(nb, nb_endpoint, normalized_data):
    '''
    :returns dict(device, msg, changed, diff): dictionary resulting of the request,
        where `device` is the serialized device fetched or newly created in
        Netbox
    '''
    data = _find_ids(nb, normalized_data)
    nb_device = nb_endpoint.get(name=data["name"])
    result = {}
    if not nb_device:
        device, diff = create_netbox_object(nb_endpoint, data, module.check_mode)
        msg = "Device %s created" % (data["name"])
        changed = True
        result["diff"] = diff
    else:
        device, diff = update_netbox_object(nb_device, data, module.check_mode)
        if device is False:
            module.fail_json(
                msg="Request failed, couldn't update device: %s" % data["name"]
            )
        if diff:
            msg = "Device %s updated" % (data["name"])
            changed = True
            result["diff"] = diff
        else:
            msg = "Device %s already exists" % (data["name"])
            changed = False
    result.update({"device": device, "changed": changed, "msg": msg})
    return result


def ensure_device_absent(nb_endpoint, data):
    '''
    :returns dict(msg, changed, diff)
    '''
    nb_device = nb_endpoint.get(name=data["name"])
    result = {}
    if nb_device:
        dummy, diff = delete_netbox_object(nb_device, module.check_mode)
        msg = 'Device %s deleted' % (data["name"])
        changed = True
        result["diff"] = diff
    else:
        msg = 'Device %s already absent' % (data["name"])
        changed = False

    result.update({"changed": changed, "msg": msg})
    return result


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}

DOCUMENTATION = """
---
module: frr_facts
version_added: "2.8"
author: "Nilashish Chakraborty (@nilashishc)"
short_description: Collect facts from remote devices running Free Range Routing (FRR).
description:
  - Collects a base set of device facts from a remote device that
    is running FRR.  This module prepends all of the
    base network fact keys with C(ansible_net_<fact>).  The facts
    module will always collect a base set of facts from the device
    and can enable or disable collection of additional facts.
notes:
  - Tested against FRR 6.0.
options:
  gather_subset:
    description:
      - When supplied, this argument restricts the facts collected
         to a given subset.
      - Possible values for this argument include
         C(all), C(hardware), C(config), and C(interfaces).
      - Specify a list of values to include a larger subset.
      - Use a value with an initial C(!) to collect all facts except that subset.
    required: false
    default: '!config'
"""

EXAMPLES = """
- name: Collect all facts from the device
  frr_facts:
    gather_subset: all

- name: Collect only the config and default facts
  frr_facts:
    gather_subset:
      - config

- name: Collect the config and hardware facts
  frr_facts:
    gather_subset:
      - config
      - hardware

- name: Do not collect hardware facts
  frr_facts:
    gather_subset:
      - "!hardware"
"""

RETURN = """
ansible_net_gather_subset:
  description: The list of fact subsets collected from the device
  returned: always
  type: list

# default
ansible_net_hostname:
  description: The configured hostname of the device
  returned: always
  type: str
ansible_net_version:
  description: The FRR version running on the remote device
  returned: always
  type: str
ansible_net_api:
  description: The name of the transport
  returned: always
  type: str
ansible_net_python_version:
  description: The Python version that the Ansible controller is using
  returned: always
  type: str

# hardware
ansible_net_mem_stats:
  description: The memory statistics fetched from the device
  returned: when hardware is configured
  type: dict

# config
ansible_net_config:
  description: The current active config from the device
  returned: when config is configured
  type: str

# interfaces
ansible_net_all_ipv4_addresses:
  description: All IPv4 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_all_ipv6_addresses:
  description: All IPv6 addresses configured on the device
  returned: when interfaces is configured
  type: list
ansible_net_interfaces:
  description: A hash of all interfaces running on the system
  returned: when interfaces is configured
  type: dict
ansible_net_mpls_ldp_neighbors:
  description: The list of MPLS LDP neighbors from the remote device
  returned: when interfaces is configured and LDP daemon is running on the device
  type: dict
"""

import platform
import re

from ansible.module_utils.network.frr.frr import run_commands, get_capabilities
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import iteritems


class FactsBase(object):

    COMMANDS = list()

    def __init__(self, module):
        self.module = module
        self.facts = dict()
        self.responses = None
        self._capabilities = get_capabilities(self.module)

    def populate(self):
        self.responses = run_commands(self.module, commands=self.COMMANDS, check_rc=False)

    def run(self, cmd):
        return run_commands(commands=cmd, check_rc=False)

    def parse_facts(self, pattern, data):
        value = None
        match = re.search(pattern, data, re.M)
        if match:
            value = match.group(1)
        return value


class Default(FactsBase):

    COMMANDS = ['show version']

    def populate(self):
        super(Default, self).populate()
        self.facts.update(self.platform_facts())

    def platform_facts(self):
        platform_facts = {}

        resp = self._capabilities
        device_info = resp['device_info']

        platform_facts['system'] = device_info['network_os']

        for item in ('version', 'hostname'):
            val = device_info.get('network_os_%s' % item)
            if val:
                platform_facts[item] = val

        platform_facts['api'] = resp['network_api']
        platform_facts['python_version'] = platform.python_version()

        return platform_facts


class Hardware(FactsBase):

    COMMANDS = ['show memory']

    def _parse_daemons(self, data):
        match = re.search(r'Memory statistics for (\w+)', data, re.M)
        if match:
            return match.group(1)

    def gather_memory_facts(self, data):
        mem_details = data.split('\n\n')
        mem_stats = {}
        mem_counters = {
            'total_heap_allocated': r'Total heap allocated:(?:\s*)(.*)',
            'holding_block_headers': r'Holding block headers:(?:\s*)(.*)',
            'used_small_blocks': r'Used small blocks:(?:\s*)(.*)',
            'used_ordinary_blocks': r'Used ordinary blocks:(?:\s*)(.*)',
            'free_small_blocks': r'Free small blocks:(?:\s*)(.*)',
            'free_ordinary_blocks': r'Free ordinary blocks:(?:\s*)(.*)',
            'ordinary_blocks': r'Ordinary blocks:(?:\s*)(.*)',
            'small_blocks': r'Small blocks:(?:\s*)(.*)',
            'holding_blocks': r'Holding blocks:(?:\s*)(.*)'
        }

        for item in mem_details:
            daemon = self._parse_daemons(item)
            mem_stats[daemon] = {}
            for fact, pattern in iteritems(mem_counters):
                mem_stats[daemon][fact] = self.parse_facts(pattern, item)

        return mem_stats

    def populate(self):
        super(Hardware, self).populate()
        data = self.responses[0]
        if data:
            self.facts['mem_stats'] = self.gather_memory_facts(data)


class Config(FactsBase):

    COMMANDS = ['show running-config']

    def populate(self):
        super(Config, self).populate()
        data = self.responses[0]
        if data:
            data = re.sub(r'^Building configuration...\s+Current configuration:', '', data, flags=re.MULTILINE)
            self.facts['config'] = data


class Interfaces(FactsBase):

    COMMANDS = ['show interface']

    def populate(self):
        ldp_supported = self._capabilities['supported_protocols']['ldp']

        if ldp_supported:
            self.COMMANDS.append('show mpls ldp discovery')

        super(Interfaces, self).populate()
        data = self.responses[0]

        self.facts['all_ipv4_addresses'] = list()
        self.facts['all_ipv6_addresses'] = list()

        if data:
            interfaces = self.parse_interfaces(data)
            self.facts['interfaces'] = self.populate_interfaces(interfaces)
            self.populate_ipv4_interfaces(interfaces)
            self.populate_ipv6_interfaces(interfaces)

        if ldp_supported:
            data = self.responses[1]
            if data:
                self.facts['mpls_ldp_neighbors'] = self.populate_mpls_ldp_neighbors(data)

    def parse_interfaces(self, data):
        parsed = dict()
        key = ''
        for line in data.split('\n'):
            if len(line) == 0:
                continue
            elif line[0] == ' ':
                parsed[key] += '\n%s' % line
            else:
                match = re.match(r'^Interface (\S+)', line)
                if match:
                    key = match.group(1)
                    parsed[key] = line
        return parsed

    def populate_interfaces(self, interfaces):
        facts = dict()
        counters = {
            'description': r'Description: (.+)',
            'macaddress': r'HWaddr: (\S+)',
            'type': r'Type: (\S+)',
            'vrf': r'vrf: (\S+)',
            'mtu': r'mtu (\d+)',
            'bandwidth': r'bandwidth (\d+)',
            'lineprotocol': r'line protocol is (\S+)',
            'operstatus': r'^(?:.+) is (.+),'
        }

        for key, value in iteritems(interfaces):
            intf = dict()
            for fact, pattern in iteritems(counters):
                intf[fact] = self.parse_facts(pattern, value)
            facts[key] = intf
        return facts

    def populate_ipv4_interfaces(self, data):
        for key, value in data.items():
            self.facts['interfaces'][key]['ipv4'] = list()
            primary_address = addresses = []
            primary_address = re.findall(r'inet (\S+) broadcast (?:\S+)(?:\s{2,})', value, re.M)
            addresses = re.findall(r'inet (\S+) broadcast (?:\S+)(?:\s+)secondary', value, re.M)
            if len(primary_address) == 0:
                continue
            addresses.append(primary_address[0])
            for address in addresses:
                addr, subnet = address.split("/")
                ipv4 = dict(address=addr.strip(), subnet=subnet.strip())
                self.add_ip_address(addr.strip(), 'ipv4')
                self.facts['interfaces'][key]['ipv4'].append(ipv4)

    def populate_ipv6_interfaces(self, data):
        for key, value in data.items():
            self.facts['interfaces'][key]['ipv6'] = list()
            addresses = re.findall(r'inet6 (\S+)', value, re.M)
            for address in addresses:
                addr, subnet = address.split("/")
                ipv6 = dict(address=addr.strip(), subnet=subnet.strip())
                self.add_ip_address(addr.strip(), 'ipv6')
                self.facts['interfaces'][key]['ipv6'].append(ipv6)

    def add_ip_address(self, address, family):
        if family == 'ipv4':
            self.facts['all_ipv4_addresses'].append(address)
        else:
            self.facts['all_ipv6_addresses'].append(address)

    def populate_mpls_ldp_neighbors(self, data):
        facts = {}
        entries = data.splitlines()
        for x in entries:
            if x.startswith('AF'):
                continue
            x = x.split()
            if len(x) > 0:
                ldp = {}
                ldp['neighbor'] = x[1]
                ldp['source'] = x[3]
                facts[ldp['source']] = []
                facts[ldp['source']].append(ldp)

        return facts


FACT_SUBSETS = dict(
    default=Default,
    hardware=Hardware,
    config=Config,
    interfaces=Interfaces
)

VALID_SUBSETS = frozenset(FACT_SUBSETS.keys())


def main():
    """main entry point for module execution
    """
    argument_spec = dict(
        gather_subset=dict(default=['!config'], type='list')
    )

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    gather_subset = module.params['gather_subset']

    runable_subsets = set()
    exclude_subsets = set()

    for subset in gather_subset:
        if subset == 'all':
            runable_subsets.update(VALID_SUBSETS)
            continue

        if subset.startswith('!'):
            subset = subset[1:]
            if subset == 'all':
                exclude_subsets.update(VALID_SUBSETS)
                continue
            exclude = True
        else:
            exclude = False

        if subset not in VALID_SUBSETS:
            module.fail_json(msg='Subset must be one of [%s], got %s' % (', '.join(VALID_SUBSETS), subset))

        if exclude:
            exclude_subsets.add(subset)
        else:
            runable_subsets.add(subset)

    if not runable_subsets:
        runable_subsets.update(VALID_SUBSETS)

    runable_subsets.difference_update(exclude_subsets)
    runable_subsets.add('default')

    facts = dict()
    facts['gather_subset'] = list(runable_subsets)

    instances = list()
    for key in runable_subsets:
        instances.append(FACT_SUBSETS[key](module))

    for inst in instances:
        inst.populate()
        facts.update(inst.facts)

    ansible_facts = dict()
    for key, value in iteritems(facts):
        key = 'ansible_net_%s' % key
        ansible_facts[key] = value

    module.exit_json(ansible_facts=ansible_facts)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_switch_controller_lldp_profile
short_description: Configure FortiSwitch LLDP profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify switch_controller feature and lldp_profile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    switch_controller_lldp_profile:
        description:
            - Configure FortiSwitch LLDP profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            802.1-tlvs:
                description:
                    - Transmitted IEEE 802.1 TLVs.
                choices:
                    - port-vlan-id
            802.3-tlvs:
                description:
                    - Transmitted IEEE 802.3 TLVs.
                choices:
                    - max-frame-size
            auto-isl:
                description:
                    - Enable/disable auto inter-switch LAG.
                choices:
                    - disable
                    - enable
            auto-isl-hello-timer:
                description:
                    - Auto inter-switch LAG hello timer duration (1 - 30 sec, default = 3).
            auto-isl-port-group:
                description:
                    - Auto inter-switch LAG port group ID (0 - 9).
            auto-isl-receive-timeout:
                description:
                    - Auto inter-switch LAG timeout if no response is received (3 - 90 sec, default = 9).
            custom-tlvs:
                description:
                    - Configuration method to edit custom TLV entries.
                suboptions:
                    information-string:
                        description:
                            - Organizationally defined information string (0 - 507 hexadecimal bytes).
                    name:
                        description:
                            - TLV name (not sent).
                        required: true
                    oui:
                        description:
                            - Organizationally unique identifier (OUI), a 3-byte hexadecimal number, for this TLV.
                    subtype:
                        description:
                            - Organizationally defined subtype (0 - 255).
            med-network-policy:
                description:
                    - Configuration method to edit Media Endpoint Discovery (MED) network policy type-length-value (TLV) categories.
                suboptions:
                    dscp:
                        description:
                            - Advertised Differentiated Services Code Point (DSCP) value, a packet header value indicating the level of service requested for
                               traffic, such as high priority or best effort delivery.
                    name:
                        description:
                            - Policy type name.
                        required: true
                    priority:
                        description:
                            - Advertised Layer 2 priority (0 - 7; from lowest to highest priority).
                    status:
                        description:
                            - Enable or disable this TLV.
                        choices:
                            - disable
                            - enable
                    vlan:
                        description:
                            - ID of VLAN to advertise, if configured on port (0 - 4094, 0 = priority tag).
            med-tlvs:
                description:
                    - "Transmitted LLDP-MED TLVs (type-length-value descriptions): inventory management TLV and/or network policy TLV."
                choices:
                    - inventory-management
                    - network-policy
            name:
                description:
                    - Profile name.
                required: true
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure FortiSwitch LLDP profiles.
    fortios_switch_controller_lldp_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      switch_controller_lldp_profile:
        state: "present"
        802.1-tlvs: "port-vlan-id"
        802.3-tlvs: "max-frame-size"
        auto-isl: "disable"
        auto-isl-hello-timer: "6"
        auto-isl-port-group: "7"
        auto-isl-receive-timeout: "8"
        custom-tlvs:
         -
            information-string: "<your_own_value>"
            name: "default_name_11"
            oui: "<your_own_value>"
            subtype: "13"
        med-network-policy:
         -
            dscp: "15"
            name: "default_name_16"
            priority: "17"
            status: "disable"
            vlan: "19"
        med-tlvs: "inventory-management"
        name: "default_name_21"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_switch_controller_lldp_profile_data(json):
    option_list = ['802.1-tlvs', '802.3-tlvs', 'auto-isl',
                   'auto-isl-hello-timer', 'auto-isl-port-group', 'auto-isl-receive-timeout',
                   'custom-tlvs', 'med-network-policy', 'med-tlvs',
                   'name']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def switch_controller_lldp_profile(data, fos):
    vdom = data['vdom']
    switch_controller_lldp_profile_data = data['switch_controller_lldp_profile']
    flattened_data = flatten_multilists_attributes(switch_controller_lldp_profile_data)
    filtered_data = filter_switch_controller_lldp_profile_data(flattened_data)
    if switch_controller_lldp_profile_data['state'] == "present":
        return fos.set('switch-controller',
                       'lldp-profile',
                       data=filtered_data,
                       vdom=vdom)

    elif switch_controller_lldp_profile_data['state'] == "absent":
        return fos.delete('switch-controller',
                          'lldp-profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_switch_controller(data, fos):
    login(data)

    if data['switch_controller_lldp_profile']:
        resp = switch_controller_lldp_profile(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "switch_controller_lldp_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "802.1-tlvs": {"required": False, "type": "str",
                               "choices": ["port-vlan-id"]},
                "802.3-tlvs": {"required": False, "type": "str",
                               "choices": ["max-frame-size"]},
                "auto-isl": {"required": False, "type": "str",
                             "choices": ["disable", "enable"]},
                "auto-isl-hello-timer": {"required": False, "type": "int"},
                "auto-isl-port-group": {"required": False, "type": "int"},
                "auto-isl-receive-timeout": {"required": False, "type": "int"},
                "custom-tlvs": {"required": False, "type": "list",
                                "options": {
                                    "information-string": {"required": False, "type": "str"},
                                    "name": {"required": True, "type": "str"},
                                    "oui": {"required": False, "type": "str"},
                                    "subtype": {"required": False, "type": "int"}
                                }},
                "med-network-policy": {"required": False, "type": "list",
                                       "options": {
                                           "dscp": {"required": False, "type": "int"},
                                           "name": {"required": True, "type": "str"},
                                           "priority": {"required": False, "type": "int"},
                                           "status": {"required": False, "type": "str",
                                                      "choices": ["disable", "enable"]},
                                           "vlan": {"required": False, "type": "int"}
                                       }},
                "med-tlvs": {"required": False, "type": "str",
                             "choices": ["inventory-management", "network-policy"]},
                "name": {"required": True, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_switch_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()
